import se.chalmers.cs.gf.translate.*;
import se.chalmers.cs.gf.util.FileUtil;
import se.chalmers.cs.gf.dialogutil.*;
import se.chalmers.cs.gf.dialogutil.sr.Recognizer;

import com.sri.oaa2.icl.*;

import java.util.*;
import java.io.IOException;

public class TranslateSpeech implements Runnable {

        private Translator translator;

        private String fromLang;

        private String toLang;

        private OAAClient oaa;

        private Recognizer sr;

        public TranslateSpeech(Translator translator, String fromLang, String toLang,
                               OAAClient oaa, boolean useSR) {
                this.translator = translator;
                this.fromLang = fromLang;
                this.toLang = toLang;
                this.oaa = oaa;
                this.sr = useSR ? new Recognizer(oaa) : null;
        }

        public void run() {
                while (true) {
                        String inputText = getTextInput();
                        System.err.println("TranslateSpeech: Text input: " + inputText);

                        Set<String> outputs = translator.translate(fromLang, toLang, inputText);

                        if (outputs.size() == 0) { // FIXME: handle these better
                                System.err.println("<no parse>");
                        } else if (outputs.size() > 1) {
                                System.err.println("<ambiguous translation>");
                                System.err.println("Possible translations: ");
                                for (String s : outputs)
                                        System.err.println(s);
                        } else {
                                String outputText = outputs.iterator().next();
                                System.err.println("TranslateSpeech: Output text: " + outputText);
                                say(toLang, outputText);
                        }
                }
        }

        private String getTextInput() {
                while (true) {
                        String s = getTextInput0();
                        if (s != null)
                                return s;
                }
        }

        private String getTextInput0() {
                if (sr == null) {
                        return javax.swing.JOptionPane.showInputDialog("Enter text");
                } else {
                        Recognizer.Result r = sr.recognize(".MAIN");
                        return r == null ? null : r.getText();
                }
        }

        private void say(String lang, String text) {
                oaa.solve(new IclStruct("speakText", new IclStr(text)));
        }

        public static void main(String [] args) throws java.io.IOException {
                String propFile = "/translate.properties";
                Properties prop = FileUtil.readPropertiesResource(TranslateSpeech.class, propFile);
                String fromLang = FileUtil.getProp(prop, "fromlang");
                String toLang = FileUtil.getProp(prop, "tolang");

                boolean useSR = args.length > 0 && args[0].equals("-sr");

                Translator t = TranslatorFactory.createTranslator(prop);

                try {
                        OAAClient oaa = new OAAClient("TranslateSpeech", args);
                        new TranslateSpeech(t, fromLang, toLang, oaa, useSR).run();
                } catch (IOException ex) {
                        System.err.println(ex);
                }
        }

}
